/*
   cfg.c -- PIKT config file routines

   $Header: /cvs/PIKT/pikt/src/piktc/cfg.c,v 1.43 2001/10/10 04:08:00 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998,1999,2000,2001 by Robert Osterlund.

   PIKT is a trademark of the University of Chicago.  All Rights Reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:  pikt-bugs@pikt.org
*/

#include "../lib/common.h"
#include "piktc.h"

int firstalarm;

/* register a macro/alarm def component
 */
void
regcomponent (ident *idp, char *component)
{
	static char fnnam[] = "regcomponent";
	def *ndfp;

	#ifdef DEEPDEBUG
	if (isdebug)
		logreturn(PIKTLOG, LOG_DEBUG, __FILE__, __LINE__, FALSE, "regcomping COMP @%s@, ident is %s", component, idp->name);
	#endif
	ndfp = (def *) mymalloc(sizeof(def), fnnam);
	ndfp->next = (def *) NULL;
	ndfp->component = component;
	if (! idp->definition)
		idp->definition = ndfp;
	else
		idp->defcurr->next = ndfp;
	idp->defcurr = ndfp;
	return;
}

/* begin writing the pikt .alt file
 */
void
beginalarms (char *alertname)
{
	static char fnnam[] = "beginalarms";
	static char *outfile;
	nam *naltp;
	char confline[MAXLINE+1];

	outfile = (char *) mymalloc((strlen(stgdir)+strlen(alertname)+6) * sizeof(char), fnnam);
	sprintf(outfile, "%s/%s.alt", stgdir, alertname);
	if ((cfgout = fopen(outfile, "w")) == NULL) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), fopen() error for %s", fnnam, outfile);
		bailout = TRUE;
		return;
	}
	free(outfile);
	naltp = (nam *) mymalloc(sizeof(nam), fnnam);
	naltp->next = (nam *) NULL;
	naltp->name = mystrdup(alertname, fnnam);
	naltp->state = ADD;
	if (! alerts)
		alerts = naltp;
	else {
		naltp->next = alerts;
		alerts = naltp;
	}
	if (nicecmd)
		sprintf(confline, "%s %s %s %s %s %s %s %s/pikt", mins, hrs, doms, mons, dows, drift, nicecmd, bindir);
	else
		sprintf(confline, "%s %s %s %s %s %s %s/pikt", mins, hrs, doms, mons, dows, drift, bindir);
	if (mailcmd) {
		strcat(confline, " +M ");
		strcat(confline, "\"");
		strcat(confline, mailcmd);
		strcat(confline, "\"");
	}
	if (lpcmd) {
		strcat(confline, " +L ");
		strcat(confline, "\"");
		strcat(confline, lpcmd);
		strcat(confline, "\"");
	}
	strcat(confline, " +A ");
	strcat(confline, alertname);
	naltp->aux = mystrdup(confline, fnnam);
	naltp->dir = (char *) NULL;
	naltp->file = (char *) NULL;
	naltp->mode = (char *) NULL;
	naltp->filuid = (char *) NULL;
	naltp->filgid = (char *) NULL;
	naltp->state = NONE;
	firstalarm = TRUE;
	return;
}

/* write an alarm definition to the pikt .alt file
 */
void
writealarm (char *alarmname)
{
	#define ISLP		(*dfp->component == '(')
	#define ISRP		(*dfp->component == ')')
	#define ISLB		(*dfp->component == '[')
	#define ISRB		(*dfp->component == ']')
	#define ISNXTLP		(dfp->next && (*dfp->next->component == '('))
	#define ISNXTRP		(dfp->next && (*dfp->next->component == ')'))
	#define ISNXTLB		(dfp->next && (*dfp->next->component == '['))
	#define ISNXTRB		(dfp->next && (*dfp->next->component == ']'))
	#define ISNXTCMA	(dfp->next && (*dfp->next->component == ','))

	static char fnnam[] = "writealarm";
	ident *idp;
	def *dfp;
	def *tdfp;
	static char *comp;
	char *cp;
	int componentlen;
	int t;			/* number of tabs */
	int isdat = FALSE;	/* is this a dat statement? */

	t = 1;			/* alarm script stanza text is always
				   indented by at least one tab
				 */
	idp = lookupident(alarmname, ALMCFG);
	if (idp) {
		if (firstalarm)
			firstalarm = FALSE;
		else
			fprintf(cfgout,"\n\n");
		fprintf(cfgout,"%s",alarmname);
		foreach (dfp, idp->definition) {
			if (isalpha((int)*dfp->component)) {
				comp = (char *) mymalloc(strlen(dfp->component)+3, fnnam);
				sprintf(comp, " %s ", dfp->component);
			} else {
				comp = (char *) NULL;
			}
			if (comp && strstr(keywords_sections, comp)) {
				t = 1;
				fprintf(cfgout, "\n\n%s%s ", tabs[t], dfp->component);
				t++;
			} else if (comp && strstr(keywords_statements, comp)) {
				if (strstr(keywords_statements_block_end, comp))
					t--;
				if (    ! strcmp(" endswitch ", comp)
				     || ! strcmp(" endsw ", comp)
				   ) {
					t--;
				}
				if ((t < 0) || (t > MAXIFS))
					logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), for %s, tab (%d) bounds exceeded (0 < t <= MAXIFS)", fnnam, alarmname);
				/* deal with 'for (' and 'switch (' special cases */
				if ((cp = strchr(comp, '\('))) {
					*(cp+1) = '\0';
					fprintf(cfgout, "\n%s%s", tabs[t], comp+1);
				} else {
					fprintf(cfgout, "\n%s%s ", tabs[t], dfp->component);
				}
				if (strstr(keywords_statements_block_begin, comp))
					t++;
				if (strstr(" dat data ", comp)) {
					isdat = TRUE;
				} else {
					if (    ! strcmp(" switch ", comp)
					        /* trailing space removed above */
					     || ! strcmp(" switch (", comp)
					   ) {
						t++;
					}
					isdat = FALSE;
				}
			} else if (ISLB) {
				fprintf(cfgout, "%s", dfp->component);
			} else if (ISLP) {
				fprintf(cfgout, "%s", dfp->component);
			} else if (ISNXTLB) {
				if (isdat) {
					/* oy vey! */
					tdfp = dfp;
					do {
						tdfp = tdfp->next;
					} while (! strchr(tdfp->component, ']'));
					tdfp = tdfp->next;
					if (     tdfp
					     &&  (    strstr(keywords_sections, tdfp->component)
						   || strstr(keywords_statements, tdfp->component)
					         )
					   ) {
						fprintf(cfgout, "%s ", dfp->component);
					} else {
						fprintf(cfgout, "%s", dfp->component);
					}
				} else {
					fprintf(cfgout, "%s", dfp->component);
				}
			} else if (ISNXTLP) {
				componentlen = strlen(dfp->component);
				if (isalpha((int)*((dfp->component)+componentlen-1)))
					fprintf(cfgout, "%s", dfp->component);
				else
					fprintf(cfgout, "%s ", dfp->component);
			} else if (ISRB && (ISNXTRB || ISNXTRP)) {
				fprintf(cfgout, "%s", dfp->component);
			} else if (ISRP && (ISNXTRB || ISNXTRP)) {
				fprintf(cfgout, "%s", dfp->component);
			} else if (ISNXTRB) {
				fprintf(cfgout, "%s", dfp->component);
			} else if (ISNXTRP) {
				fprintf(cfgout, "%s", dfp->component);
			} else if (ISNXTCMA) {
				fprintf(cfgout, "%s", dfp->component);
			} else {
				fprintf(cfgout, "%s ", dfp->component);
			}
			FREE(comp);
		}
	} else {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), %s, %s alarm not registered", fnnam, cfgfilecurr, alarmname);
		bailout = TRUE;
		return;
	}
	return; 
}

/* close the pikt .alt file
 */
void
endalarms (char *alert)
{
	static char fnnam[] = "endalarms";

	fprintf(cfgout,"\n");
	if (fclose(cfgout)) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), fclose() error for %s/%s.alt", fnnam, altdir, alert);
		bailout = TRUE;
		return;
	}
	return;
}

/* begin writing the other/program file
 */
void
beginfile (char *name, char *filepath, char *mode, char *filuid, char *filgid, nam **list)
{
	static char fnnam[] = "beginfile";
	char *cp;
	char *dir, *file;
	static char *outfile;
	static char *diffile;
	nam *nfilp;

	if ((cp = strrchr(filepath, '/'))) {
		*cp = '\0';
		dir = mystrdup(filepath, fnnam);
		file = mystrdup(cp+1, fnnam);
		outfile = (char *) mymalloc((strlen(stgdir)+strlen(dir)+strlen(file)+2) * sizeof(char), fnnam);
		sprintf(outfile, "%s%s/%s", stgdir, dir, file);
		/* make the staging subdirs for this file */
		mkdirtree(outfile);
		if (isdiff) {
			diffile = (char *) mymalloc((strlen(difdir)+strlen(dir)+strlen(file)+2) * sizeof(char), fnnam);
			sprintf(diffile, "%s%s/%s", difdir, dir, file);
			/* make the diffing subdirs for this file */
			mkdirtree(diffile);
			free(diffile);
		}
	} else {
		dir = mystrdup("", fnnam);
		file = mystrdup(filepath, fnnam);
		outfile = (char *) mymalloc((strlen(stgdir)+strlen(file)+2) * sizeof(char), fnnam);
		sprintf(outfile, "%s/%s", stgdir, file);
	}
	if ((cfgout = fopen(outfile, "w")) == NULL) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), fopen() error for %s", fnnam, outfile);
		bailout = TRUE;
		return;
	}
	free(outfile);
	nfilp = (nam *) mymalloc(sizeof(nam), fnnam);
	nfilp->next = (nam *) NULL;
	if (*name == '/') {
		*cp = '/';	/* restore the '/' we blotted out above */
		if (strcmp(name, filepath)) {
			logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), ident %s doesn't match filepath %s", fnnam, name, filepath);
			bailout = TRUE;
			return;
		} else {
			nfilp->name = mystrdup(file, fnnam);
		}
	} else {
		nfilp->name = mystrdup(name, fnnam);
	}
	nfilp->aux = (char *) NULL;
	nfilp->dir = dir;
	nfilp->file = file;
	nfilp->mode = mode;
	nfilp->filuid = filuid;
	nfilp->filgid = filgid;
	nfilp->state = ADD;
	if (*list)
		nfilp->next = *list;
	*list = nfilp;
	return;
}

/* write a single line to the other/program file
 */
void
writefile (char *line)
{

	fprintf(cfgout, "%s\n", line);
	return;
}

/* close the other/program file
 */
void
endfile (char *file)
{
	static char fnnam[] = "endfile";
	char *cp;

	if ((cp = strrchr(file, '/')))
		cp = cp+1;
	else
		cp = file;
	if (fclose(cfgout)) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), fclose() error for %s/%s", fnnam, stgdir, cp);
		bailout = TRUE;
		return;
	}
	return;
}

/* begin writing the objectfile
 */
void
beginobjects (char *objectname)
{
	static char fnnam[] = "beginobjects";
	static char *outfile;
	nam *nobjp;

	outfile = (char *) mymalloc((strlen(stgdir)+strlen(objectname)+6) * sizeof(char), fnnam);
	sprintf(outfile, "%s/%s.obj", stgdir, objectname);
	if ((cfgout = fopen(outfile, "w")) == NULL) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), fopen() error for %s", fnnam, outfile);
		bailout = TRUE;
		return;
	}
	free(outfile);
	nobjp = (nam *) mymalloc(sizeof(nam), fnnam);
	nobjp->next = (nam *) NULL;
	nobjp->name = mystrdup(objectname, fnnam);
	nobjp->aux = (char *) NULL;
	nobjp->dir = (char *) NULL;
	nobjp->file = (char *) NULL;
	nobjp->mode = (char *) NULL;
	nobjp->filuid = (char *) NULL;
	nobjp->filgid = (char *) NULL;
	nobjp->state = ADD;
	if (! objects)
		objects = nobjp;
	else {
		nobjp->next = objects;
		objects = nobjp;
	}

}

/* write a single object to the objectfile
 */
void
writeobject (char *object)
{

	fprintf(cfgout,"%s\n",object);
}

/* close the objectfile
 */
void
endobjects (char *objs)
{
	static char fnnam[] = "endobjects";

	if (fclose(cfgout)) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), fclose error for %s/%s.obj", fnnam, objdir, objs);
		bailout = TRUE;
		return;
	}
	return;
}

/* return the length of the piktd.conf line for the given alert
 */
int
conflinelen (char *alertname)
{
	int len;

	len = strlen(pikt)+1;			/* e.g., '/pikt/bin/pikt ' */
	len += strlen(mins) + 1 +		/* 'x ' */
	       strlen(hrs) + 1 +		/* 'x ' */
	       strlen(doms) + 1 +		/* 'x ' */
	       strlen(mons) + 1 +		/* 'x ' */
	       strlen(dows) + 1 ;		/* 'x ' */
	if (nicecmd)
		len += 1 + strlen(nicecmd);
	if (mailcmd)
		len += 6 + strlen(mailcmd);	/* '+M "..." ' */
	if (lpcmd)
		len += 6 + strlen(lpcmd);	/* '+P "..." ' */
	len += 4 + strlen(alertname);		/* '+A ...\n' */
	return len;
}

/* given a pathname, make (as necessary) all directories in that path
 */
void
mkdirtree (char *path)
{
	static char fnnam[] = "mkdirtree";
	register int i;
	register char c;

	i = 0;
	if (*(path+i++) == '/') {
		while ((c = *(path+i))) {
			if (c == '/') {
				*(path+i) = '\0';
				if (! isfile(path, FALSE)) {
					if (mkdir(path, (mode_t)(S_IRWXU|S_IRGRP|S_IWGRP)) == -1) {	/* 750 */
						logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in %s(), mkdir() failure for %s", fnnam, path);
						bailout = TRUE;
						return;
					}
				}
				*(path+i) = '/';
			}
			i++;
		}
	} else {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in %s(), path %s is not an absolute path", fnnam, path);
		bailout = TRUE;
		return;
	}
	return;
}
