%{

/*
   keys.y -- PIKT parser for the keys.conf file

   $Header: /cvs/PIKT/pikt/src/piktc/keys.y,v 1.2 2001/10/10 04:08:00 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998,1999,2000,2001 by Robert Osterlund.

   PIKT is a trademark of the University of Chicago.  All Rights Reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   Send problem reports, bouquets and brickbats to:  pikt-bugs@pikt.org
 */

#include "../lib/common.h"
#include "piktc.h"

ident *hidp;

%}

%union {
	int ival;
	char *sval;
}

%token <sval> _HOST
%token <sval> _STR
%token <ival> _INT

%%

start:		conflines
		;

conflines:	confline
		| conflines confline
		;

confline:  	_HOST _INT _INT {
		  if ((hidp = lookupident($1, SYSCFG))) {
			hidp->piktuid = $2;
			hidp->piktgid = $3;
			if (hostmatch($1, master)) {
				authuidmaster = (AUTHUNIX_CREATE_UID_T) $2;
				authgidmaster = (AUTHUNIX_CREATE_GID_T) $3;
			}
		  } else {
                  	logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in keysparse(), for keys.conf, %s not registered as host in systems.cfg", $1);
		  }
		}
		| _HOST _STR {
		  if ((hidp = lookupident($1, SYSCFG))) {
			free(hidp->piktprivate_key);
			hidp->piktprivate_key = $2;
			if (hostmatch($1, master)) {
				pvtkeymaster = $2;
			}
			if (strlen($2) > MAXKEYBYTES) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in keys.conf, for host %s, piktprivate_key %s, key length exceeds maximum (%d), specify another piktprivate_key", $1, $2, MAXKEYBYTES);
				bailout = TRUE;
			}
		  } else {
                  	logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in keysparse(), for keys.conf, %s not registered as host in systems.cfg", $1);
		  }
		}
		| _HOST _INT _INT _STR {
		  if ((hidp = lookupident($1, SYSCFG))) {
			hidp->piktuid = $2;
			hidp->piktgid = $3;
			hidp->piktprivate_key = $4;
			if (hostmatch($1, master)) {
				authuidmaster = (AUTHUNIX_CREATE_UID_T) $2;
				authgidmaster = (AUTHUNIX_CREATE_GID_T) $3;
				pvtkeymaster = $4;
			}
			if (strlen($4) > MAXKEYBYTES) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in keys.conf, for host %s, piktprivate_key %s, key length exceeds maximum (%d), specify another piktprivate_key", $1, $4, MAXKEYBYTES);
				bailout = TRUE;
			}
		  } else {
                  	logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in keysparse(), for keys.conf, %s not registered as host in systems.cfg", $1);
		  }
		}
		;

%%
